import chalk from 'chalk';

const ACTION_STATUS = {
  NOT_STARTED: 0,
  IN_PROGRESS: 1,
  COMPLETE_SUCCESS: 2,
  COMPLETE_CONTINUE: 3,
  COMPLETE_FAILURE: 4
};

class Action {
  #_status;
  #_steps;

  constructor(actionTitle) {
      this.actionTitle = actionTitle;
      this.#_status = ACTION_STATUS.NOT_STARTED;
      this.#_steps = [];
  }

  printStatus() {
      switch (this.status) {
          case ACTION_STATUS.NOT_STARTED:
              console.log(`${chalk.gray(this.actionTitle)}`);
              break;
          case ACTION_STATUS.IN_PROGRESS:
              console.log(`${chalk.yellow(this.actionTitle)}`);
              break;
          case ACTION_STATUS.COMPLETE_SUCCESS:
              console.log(`${chalk.green(this.actionTitle)}`);
              break;
          case ACTION_STATUS.COMPLETE_FAILURE:
              console.log(`${chalk.red(this.actionTitle)}`);
              break;
      }
  }

  execute() {
      this.#_steps.forEach((step) => {
          try {
              switch (step()) {
                  case ACTION_STATUS.COMPLETE_SUCCESS:
                      this.status = ACTION_STATUS.COMPLETE_SUCCESS;
                      break;
                  case ACTION_STATUS.COMPLETE_FAILURE:
                      this.status = ACTION_STATUS.COMPLETE_FAILURE;
                      break;
                  case ACTION_STATUS.COMPLETE_CONTINUE:
                      this.status = ACTION_STATUS.IN_PROGRESS;
                      break;
              }
          } catch (e) {
              this.status = ACTION_STATUS.COMPLETE_FAILURE;
          }

          if (this.status == ACTION_STATUS.COMPLETE_FAILURE || this.status == ACTION_STATUS.COMPLETE_SUCCESS) {
              return;
          }
      });
  }

  set status(status) {
      this.#_status = status;
  }

  get status() {
      return this.#_status;
  }

  addStep(func) {
      this.#_steps.push(func);
  }
}

export { Action, ACTION_STATUS };