'use strict';

import chalk from 'chalk';
import { Command, Argument } from 'commander';
import packageJson from '../package.json' assert { type: "json" };
import fs from 'fs';
import path from 'path';
import { fileURLToPath } from 'url';
const __dirname = fileURLToPath(new URL('../..', import.meta.url));

class BootstrapToolsCommand extends Command {
  constructor(name) {
    super(name);

    this.option('--dry-run', 'display command outputs without actually performing any actions.');
  }

  async initCommands(packageDir) {
    let dirs = fs.readdirSync(packageDir);

    for await (const dir of dirs) {
      if (fs.statSync(`${packageDir}/${dir}`).isDirectory()) {
        const cmdPackageJson = (await import(`${packageDir}/${dir}/package.json`, {
          assert: { type: "json" }
        })).default;

        try {
          if (cmdPackageJson.name !== packageJson.name) {
            const cmd = (await import(cmdPackageJson.name)).default;
            let bootstrapToolCommand = new cmd(dir);
            this.addCommand(bootstrapToolCommand);
          }
        }
        catch (e) {
          console.log(`${dir} is not a Command class.`);
          console.error(e);
        }
      }
    }
  }
};

async function buildCommanderProgram() {
  const program = new BootstrapToolsCommand('bootstrap-tool');
  await program.initCommands(__dirname);

  // program
  //   .name(packageJson.name)
  //   .version(packageJson.version)
  //   .hook('preAction', (thisCommand, actionCommand) => {
  //     if (thisCommand.opts().trace) {
  //       console.log(`About to call action handler for subcommand: ${actionCommand.name()}`);
  //       console.log('arguments: %O', actionCommand.args);
  //       console.log('options: %o', actionCommand.opts());
  //     }
  //   });

  // program.addCommand(buildCommand());

  // program
  //   .command('create-docker-baseline')
  //   .description('setup a new docker-compose configuration base on a solution template')
  //   .argument('<dir>', 'directory to setup compose and volumees template')
  //   .option('-t, --template <template>', 'template to use')
  //   .allowUnknownOption(true)
  //   .action((...args) => {
  //     const { invoke } = require(`@kickstep/create-docker-baseline`);
  //     let commandArgs = args.splice(0, args.length - 1);
  //     invoke(commandArgs);
  //   });

  return program;
}

export default buildCommanderProgram;